#
# hierarchygraph tree navigator for nebula based on bwidget treedemo
# copyright ghoust@gmx.de/tom@3d-inferno.com Thomas Miskiewicz
# Visit 3d-inferno.com for updates or new plugin scripts
#
# 
# You may use this script to your need as long as you leave this copyright
# in place (general copy left license NOT GNU copy left license)
# No responibility taken for any harm caused by this script, use at own risk. Provided as is.

namespace eval NavTree {
    variable count
    variable dblclick
    #path to node bitmaps
    variable imgpath
    variable vars
    #stores the found node types
    variable nodes
    #path to node scripts
    variable nodepath

    set base [file dirname [info script]]
    set nodepath "$base/nodes"
    set imgpath "$base/bmp"
    set opath [pwd]
    cd $nodepath
    set items [glob -nocomplain *.tcl]
    cd $opath

    #precache plugins !!
    foreach script $items {
	namespace inscope :: source $nodepath/$script
	lappend nodes [file rootname $script]
    }

}


proc NavTree::create { nb } {

    set pw    [PanedWindow $nb.pw -side top]

    set pane  [$pw add -weight 1]
    set title [TitleFrame $pane.lf -text "Hierarchygraph" -side left]
    set sw    [ScrolledWindow [$title getframe].sw \
                  -relief sunken -borderwidth 2]
    set tree  [Tree $sw.tree \
                   -relief flat -borderwidth 0 -width 15 -highlightthickness 0\
		   -redraw 0 -dropenabled 0 -dragenabled 0 \
                   -opencmd   "NavTree::moddir 1 $sw.tree" \
                   -closecmd  "NavTree::moddir 0 $sw.tree"]
    $sw setwidget $tree

    pack $sw    -side left -expand yes -fill both
    pack $title -fill both -expand yes

    set pane [$pw add -weight 2]
    set lf   [TitleFrame $pane.lf -text "Properties" -side right]
    set sw   [ScrolledWindow [$lf getframe].sw \
                  -relief flat -borderwidth 2]
    set can [canvas $sw.can -relief flat -width 80 -height 300]
    $sw setwidget $can

    pack $sw $lf -side right -fill both -expand yes
    pack $pw -side right -fill both -expand yes

    $tree bindText  <ButtonPress-1>        "NavTree::select tree 1 $tree "
    $tree bindText  <Double-ButtonPress-1> "NavTree::select tree 2 $tree "
    
    NavTree::init $tree $sw
}


proc NavTree::init { tree args } {
    global   tcl_platform
    variable count
    variable imgpath
    variable vars

    set count 0
    set vars $args
    $tree insert end root home -text "/" -data "/" -open 1 \
        -image [Bitmap::get  "$imgpath/general" ]
    getdir $tree home "/"
    NavTree::select tree 1 $tree home
    $tree configure -redraw 1

    # initial information
   label $vars.f10 -text "Please select a node and then double click \
on it to view and adjust the values of the node. Feedback apreciated \
by tom@3d-inferno.com" -wraplength 80
   pack $vars.f10 -side top -padx 2 -expand yes -fill x
}


proc NavTree::getdir { tree node path } {
    variable count
    variable imgpath

    set act [psel]
    sel $path
    set lentries [dir]
    set lfiles   {}

    set img " "
    foreach f $lentries {
           if { [file exists "$imgpath/[$path/$f.getclass].gif" ] } {
           set img  "$imgpath/[$path/$f.getclass].gif" 
           } else {
           set img  "$imgpath/general" 
            }
            $tree insert end $node n:$count \
               -text "[$path/$f.getclass]  | $f"  \
                -image     [Bitmap::get $img ] \
                -drawcross allways \
                -data      "$path/$f"
            incr count
    }
    $tree itemconfigure $node -drawcross auto
    sel $act
}


proc NavTree::moddir { idx tree node } {
  if { $idx && [$tree itemcget $node -drawcross] == "allways" } {
        getdir $tree $node [$tree itemcget $node -data]
  }
  if { !$idx } {
       set desc [$tree nodes $node]
       set val [$tree itemcget $node -data]
       $tree delete $desc
       $tree itemconfigure $node -drawcross allways
    NavTree::select tree 1 $tree $node
    $tree configure -redraw 1
  }
}


proc NavTree::select { where num tree node } {
    variable dblclick

    set dblclick 1
    if { $num == 1 } {
        if { $where == "tree" && [lsearch [$tree selection get] $node] != -1 } {
            unset dblclick
            after 500 "NavTree::edit tree $tree $node"
            return
        }
        if { $where == "tree" } {
            select_node $tree $node
        } 
    }
}


proc NavTree::select_node { tree node } {
    variable vars

    $tree selection set $node
    update

    if { [$tree itemcget $node -drawcross] == "allways" } {
        getdir $tree $node [$tree itemcget $node -data]
    }
}


proc NavTree::edit { where tree node } {
    variable vars
    variable nodes

    #delete old items
    set o [pack slaves $vars]
    foreach old $o {
      destroy $old
    }

    #show new items from plugins
    set cl [[$tree itemcget $node -data].getclass]
    if { [lsearch $nodes $cl] != -1 } {
      set o "${cl}::read"
      eval $o $vars "[$tree itemcget $node -data]"
    } else {
      nroot::read $vars "[$tree itemcget $node -data]"
    }
}


proc NavTree::expand { tree but } {
    if { [set cur [$tree selection get]] != "" } {
        if { $but == 0 } {
            $tree opentree $cur
        } else {
            $tree closetree $cur
        }
    }
}


